// COMClient.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <ATLComTime.h>
#include <ATLSafe.h>
#include <iostream>
#include <fstream>

//#import "progid:COMplus.COMplusObject.1" \
//	no_namespace no_smart_pointers raw_interfaces_only
//#import "progid:DCOMtest.DCOMObject.1" \
//	no_namespace no_smart_pointers raw_interfaces_only
#import "..\COMplus\_COMplus.tlb"   no_namespace no_smart_pointers raw_interfaces_only
#import "..\DCOMTest\_DCOMtest.tlb" no_namespace no_smart_pointers raw_interfaces_only
#ifdef _DEBUG
#import "..\NETCOMplus\bin\Debug\NETCOMplus.tlb" \
	no_namespace no_smart_pointers raw_interfaces_only
#else
#import "..\NETCOMplus\bin\Release\NETCOMplus.tlb" \
	no_namespace no_smart_pointers raw_interfaces_only
#endif

#pragma warning(disable:4311)
#pragma warning(disable:4312)

const int
	IterationsNumber = 1000,
	WarmingNumber = 5,
	ThreadsNumber = 10,
	ArraySize = 100;
const BSTR
	TestString = L"0123456789",
	ServerName = L"localhost";

bool local = false;
std::ofstream logfile;
CComCriticalSection csec;

COleDateTimeSpan span[ThreadsNumber];

CComSafeArray<LONG> intArr;
CComSafeArray<BSTR> strArr;

void CreateArrays()
{
	intArr.Create(ArraySize);	
	for (int i=0; i<ArraySize; i++)
		intArr.SetAt(i,i);

	strArr.Create(ArraySize);	
	for (int i=0; i<ArraySize; i++)
		strArr.SetAt(i,TestString);
}

IUnknown *CreateObject(const CLSID& rclsid,const IID& riid)
{
	IUnknown *p = NULL;
	HRESULT hr;

	if (local) 
	{
		hr = ::CoCreateInstance(rclsid,NULL,CLSCTX_SERVER,riid,(void**)&p);
	}
	else 
	{
		COSERVERINFO csi = { 0, ServerName, 0, 0 };
		MULTI_QI mqi = { &riid, 0, 0 };
		hr = ::CoCreateInstanceEx(rclsid,0,CLSCTX_SERVER,&csi,1,&mqi);
		p = mqi.pItf;
	}

	if (FAILED(hr))
	{
		std::cout << "Error: HRESULT = 0x" << std::hex << hr << std::endl;
		throw (int)hr;
	}

	return p;
}

void CoInit()
{
	::CoInitialize(NULL);

	::CoInitializeSecurity(
		0, -1, 0, 0,
		RPC_C_AUTHN_LEVEL_CONNECT,
		RPC_C_IMP_LEVEL_IDENTIFY,
		0, 0, 0 );
}

void CoUnint()
{
	::CoUninitialize();
}


/// --- ICOMplusObject -----------------------------------------

ICOMplusObject *CreateCOMplusObject()
{
	return (ICOMplusObject*)
		CreateObject(__uuidof(CCOMplusObject),__uuidof(ICOMplusObject));
}

DWORD WINAPI COMplusObjectMethod1Call(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	ICOMplusObject *p = CreateCOMplusObject();
	for (int i=0; i<IterationsNumber; i++)
	{
		p->Method1();
	}
	p->Release();
	CoUnint();
	span[(long)n] = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

DWORD WINAPI COMplusObjectMethod1Connect(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	for (int i=0; i<IterationsNumber; i++)
	{
		ICOMplusObject *p = CreateCOMplusObject();
		p->Method1();
		p->Release();
	}
	CoUnint();
	span[(long)n] = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

DWORD WINAPI COMplusObjectMethod2Call(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	ICOMplusObject *p = CreateCOMplusObject();
	LONG par = 0, ret;
	for (int i=0; i<IterationsNumber; i++)
	{
		p->Method2(par,&ret);
	}
	p->Release();
	CoUnint();
	span[(long)n] = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

DWORD WINAPI COMplusObjectMethod2Connect(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	LONG par = 0, ret;
	for (int i=0; i<IterationsNumber; i++)
	{
		ICOMplusObject *p = CreateCOMplusObject();
		p->Method2(par,&ret);
		p->Release();
	}
	CoUnint();
	span[(long)n] = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

DWORD WINAPI COMplusObjectMethod3Call(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	ICOMplusObject *p = CreateCOMplusObject();
	CComBSTR par = TestString;
	for (int i=0; i<IterationsNumber; i++)
	{
		CComBSTR ret;
		p->Method3(par,&ret);
	}
	p->Release();
	CoUnint();
	span[(long)n]  = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

DWORD WINAPI COMplusObjectMethod3Connect(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	CComBSTR par = TestString;
	for (int i=0; i<IterationsNumber; i++)
	{
		CComBSTR ret;
		ICOMplusObject *p = CreateCOMplusObject();
		p->Method3(par,&ret);
		p->Release();
	}
	CoUnint();
	span[(long)n]  = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

DWORD WINAPI COMplusObjectMethod4Call(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	ICOMplusObject *p = CreateCOMplusObject();
	for (int i=0; i<IterationsNumber; i++)
	{
		CComVariant ret;
		p->Method4(CComVariant(intArr),&ret);
	}
	p->Release();
	CoUnint();
	span[(long)n]  = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

DWORD WINAPI COMplusObjectMethod4Connect(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	for (int i=0; i<IterationsNumber; i++)
	{
		CComVariant ret;
		ICOMplusObject *p = CreateCOMplusObject();
		p->Method4(CComVariant(intArr),&ret);
		p->Release();
	}
	CoUnint();
	span[(long)n]  = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

DWORD WINAPI COMplusObjectMethod5Call(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	ICOMplusObject *p = CreateCOMplusObject();
	for (int i=0; i<IterationsNumber; i++)
	{
		CComVariant ret;
		p->Method4(CComVariant(strArr),&ret);
	}
	p->Release();
	CoUnint();
	span[(long)n]  = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

DWORD WINAPI COMplusObjectMethod5Connect(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	for (int i=0; i<IterationsNumber; i++)
	{
		CComVariant ret;
		ICOMplusObject *p = CreateCOMplusObject();
		p->Method4(CComVariant(strArr),&ret);
		p->Release();
	}
	CoUnint();
	span[(long)n]  = COleDateTime::GetCurrentTime() - begin;
	return 0;
}


/// --- ICOMplusPooledObject -----------------------------------

ICOMplusPooledObject *CreateCOMplusPooledObject()
{
	return (ICOMplusPooledObject*)
        CreateObject(__uuidof(CCOMplusPooledObject),__uuidof(ICOMplusPooledObject));
}

DWORD WINAPI COMplusPooledObjectMethod1Call(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	ICOMplusPooledObject *p = CreateCOMplusPooledObject();
	for (int i=0; i<IterationsNumber; i++)
	{
		p->Method1();
	}
	p->Release();
	CoUnint();
	span[(long)n]  = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

DWORD WINAPI COMplusPooledObjectMethod1Connect(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	for (int i=0; i<IterationsNumber; i++)
	{
		ICOMplusPooledObject *p = CreateCOMplusPooledObject();
		p->Method1();
		p->Release();
	}
	CoUnint();
	span[(long)n]  = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

DWORD WINAPI COMplusPooledObjectMethod2Call(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	ICOMplusPooledObject *p = CreateCOMplusPooledObject();
	LONG par = 0, ret;
	for (int i=0; i<IterationsNumber; i++)
	{
		p->Method2(par,&ret);
	}
	p->Release();
	CoUnint();
	span[(long)n]  = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

DWORD WINAPI COMplusPooledObjectMethod2Connect(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	LONG par = 0, ret;
	for (int i=0; i<IterationsNumber; i++)
	{
		ICOMplusPooledObject *p = CreateCOMplusPooledObject();
		p->Method2(par,&ret);
		p->Release();
	}
	CoUnint();
	span[(long)n]  = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

DWORD WINAPI COMplusPooledObjectMethod3Call(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	ICOMplusPooledObject *p = CreateCOMplusPooledObject();
	CComBSTR par = TestString;
	for (int i=0; i<IterationsNumber; i++)
	{
		CComBSTR ret;
		p->Method3(par,&ret);
	}
	p->Release();
	CoUnint();
	span[(long)n]  = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

DWORD WINAPI COMplusPooledObjectMethod3Connect(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	CComBSTR par = TestString;
	for (int i=0; i<IterationsNumber; i++)
	{
		CComBSTR ret;
		ICOMplusPooledObject *p = CreateCOMplusPooledObject();
		p->Method3(par,&ret);
		p->Release();
	}
	CoUnint();
	span[(long)n]  = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

DWORD WINAPI COMplusPooledObjectMethod4Call(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	ICOMplusPooledObject *p = CreateCOMplusPooledObject();
	for (int i=0; i<IterationsNumber; i++)
	{
		CComVariant ret;
		p->Method4(CComVariant(intArr),&ret);
	}
	p->Release();
	CoUnint();
	span[(long)n]  = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

DWORD WINAPI COMplusPooledObjectMethod4Connect(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	for (int i=0; i<IterationsNumber; i++)
	{
		CComVariant ret;
		ICOMplusPooledObject *p = CreateCOMplusPooledObject();
		p->Method4(CComVariant(intArr),&ret);
		p->Release();
	}
	CoUnint();
	span[(long)n]  = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

DWORD WINAPI COMplusPooledObjectMethod5Call(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	ICOMplusPooledObject *p = CreateCOMplusPooledObject();
	for (int i=0; i<IterationsNumber; i++)
	{
		CComVariant ret;
		p->Method4(CComVariant(strArr),&ret);
	}
	p->Release();
	CoUnint();
	span[(long)n] = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

DWORD WINAPI COMplusPooledObjectMethod5Connect(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	for (int i=0; i<IterationsNumber; i++)
	{
		CComVariant ret;
		ICOMplusPooledObject *p = CreateCOMplusPooledObject();
		p->Method4(CComVariant(strArr),&ret);
		p->Release();
	}
	CoUnint();
	span[(long)n] = COleDateTime::GetCurrentTime() - begin;
	return 0;
}


/// --- IDCOMObject --------------------------------------------

IDCOMObject *CreateDCOMObject()
{
	return (IDCOMObject*)
		CreateObject(__uuidof(CDCOMObject),__uuidof(IDCOMObject));
}

DWORD WINAPI DCOMObjectMethod1Call(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	IDCOMObject *p = CreateDCOMObject();
	for (int i=0; i<IterationsNumber; i++)
	{
		p->Method1();
	}
	p->Release();
	CoUnint();
	span[(long)n] = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

DWORD WINAPI DCOMObjectMethod1Connect(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	for (int i=0; i<IterationsNumber; i++)
	{
		IDCOMObject *p = CreateDCOMObject();
		p->Method1();
		p->Release();
	}
	CoUnint();
	span[(long)n] = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

DWORD WINAPI DCOMObjectMethod2Call(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	IDCOMObject *p = CreateDCOMObject();
	LONG par = 0, ret;
	for (int i=0; i<IterationsNumber; i++)
	{
		p->Method2(par,&ret);
	}
	p->Release();
	CoUnint();
	span[(long)n] = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

DWORD WINAPI DCOMObjectMethod2Connect(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	LONG par = 0, ret;
	for (int i=0; i<IterationsNumber; i++)
	{
		IDCOMObject *p = CreateDCOMObject();
		p->Method2(par,&ret);
		p->Release();
	}
	CoUnint();
	span[(long)n] = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

DWORD WINAPI DCOMObjectMethod3Call(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	IDCOMObject *p = CreateDCOMObject();
	CComBSTR par = TestString;
	for (int i=0; i<IterationsNumber; i++)
	{
		CComBSTR ret;
		p->Method3(par,&ret);
	}
	p->Release();
	CoUnint();
	span[(long)n] = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

DWORD WINAPI DCOMObjectMethod3Connect(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	CComBSTR par = TestString;
	for (int i=0; i<IterationsNumber; i++)
	{
		CComBSTR ret;
		IDCOMObject *p = CreateDCOMObject();
		p->Method3(par,&ret);
		p->Release();
	}
	CoUnint();
	span[(long)n] = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

DWORD WINAPI DCOMObjectMethod4Call(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	IDCOMObject *p = CreateDCOMObject();
	for (int i=0; i<IterationsNumber; i++)
	{
		CComVariant ret;
		p->Method4(CComVariant(intArr),&ret);
	}
	p->Release();
	CoUnint();
	span[(long)n] = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

DWORD WINAPI DCOMObjectMethod4Connect(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	for (int i=0; i<IterationsNumber; i++)
	{
		CComVariant ret;
		IDCOMObject *p = CreateDCOMObject();
		p->Method4(CComVariant(intArr),&ret);
		p->Release();
	}
	CoUnint();
	span[(long)n] = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

DWORD WINAPI DCOMObjectMethod5Call(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	IDCOMObject *p = CreateDCOMObject();
	for (int i=0; i<IterationsNumber; i++)
	{
		CComVariant ret;
		p->Method4(CComVariant(strArr),&ret);
	}
	p->Release();
	CoUnint();
	span[(long)n] = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

DWORD WINAPI DCOMObjectMethod5Connect(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	for (int i=0; i<IterationsNumber; i++)
	{
		CComVariant ret;
		IDCOMObject *p = CreateDCOMObject();
		p->Method4(CComVariant(strArr),&ret);
		p->Release();
	}
	CoUnint();
	span[(long)n] = COleDateTime::GetCurrentTime() - begin;
	return 0;
}


/// --- INETObject ---------------------------------------------

INETObject *CreateNETObject()
{
	return (INETObject*)
        CreateObject(__uuidof(NETObject),__uuidof(INETObject));
}

DWORD WINAPI NETObjectMethod1Call(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	INETObject *p = CreateNETObject();
	for (int i=0; i<IterationsNumber; i++)
	{
		p->Method1();
	}
	p->Release();
	CoUnint();
	span[(long)n] = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

DWORD WINAPI NETObjectMethod1Connect(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	for (int i=0; i<IterationsNumber; i++)
	{
		INETObject *p = CreateNETObject();
		p->Method1();
		p->Release();
	}
	CoUnint();
	span[(long)n] = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

DWORD WINAPI NETObjectMethod2Call(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	INETObject *p = CreateNETObject();
	LONG par = 0, ret;
	for (int i=0; i<IterationsNumber; i++)
	{
		p->Method2(par,&ret);
	}
	p->Release();
	CoUnint();
	span[(long)n] = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

DWORD WINAPI NETObjectMethod2Connect(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	LONG par = 0, ret;
	for (int i=0; i<IterationsNumber; i++)
	{
		INETObject *p = CreateNETObject();
		p->Method2(par,&ret);
		p->Release();
	}
	CoUnint();
	span[(long)n] = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

DWORD WINAPI NETObjectMethod3Call(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	INETObject *p = CreateNETObject();
	CComBSTR par = TestString;
	for (int i=0; i<IterationsNumber; i++)
	{
		CComBSTR ret;
		p->Method3(par,&ret);
	}
	p->Release();
	CoUnint();
	span[(long)n] = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

DWORD WINAPI NETObjectMethod3Connect(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	CComBSTR par = TestString;
	for (int i=0; i<IterationsNumber; i++)
	{
		CComBSTR ret;
		INETObject *p = CreateNETObject();
		p->Method3(par,&ret);
		p->Release();
	}
	CoUnint();
	span[(long)n] = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

DWORD WINAPI NETObjectMethod4Call(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	INETObject *p = CreateNETObject();
	for (int i=0; i<IterationsNumber; i++)
	{
		CComVariant ret;
		p->Method4(CComVariant(intArr),&ret);
	}
	p->Release();
	CoUnint();
	span[(long)n] = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

DWORD WINAPI NETObjectMethod4Connect(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	for (int i=0; i<IterationsNumber; i++)
	{
		CComVariant ret;
		INETObject *p = CreateNETObject();
		p->Method4(CComVariant(intArr),&ret);
		p->Release();
	}
	CoUnint();
	span[(long)n] = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

DWORD WINAPI NETObjectMethod5Call(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	INETObject *p = CreateNETObject();
	for (int i=0; i<IterationsNumber; i++)
	{
		CComVariant ret;
		p->Method4(CComVariant(strArr),&ret);
	}
	p->Release();
	CoUnint();
	span[(long)n] = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

DWORD WINAPI NETObjectMethod5Connect(LPVOID n)
{
	COleDateTime begin = COleDateTime::GetCurrentTime();
	CoInit();
	for (int i=0; i<IterationsNumber; i++)
	{
		CComVariant ret;
		INETObject *p = CreateNETObject();
		p->Method4(CComVariant(strArr),&ret);
		p->Release();
	}
	CoUnint();
	span[(long)n] = COleDateTime::GetCurrentTime() - begin;
	return 0;
}

void RunMethod(LPTHREAD_START_ROUTINE method)
{
	HANDLE th[ThreadsNumber];
	memset(th,0,sizeof th);

	COleDateTime begin = COleDateTime::GetCurrentTime();

	for (long i=0; i<ThreadsNumber; i++)
        th[i] = ::CreateThread(NULL,0,method,(LPVOID)i,0,NULL);
	::WaitForMultipleObjects(ThreadsNumber,th,TRUE,INFINITE);

	logfile << (LPCTSTR)(COleDateTime::GetCurrentTime() - begin).Format(" %H:%M:%S:");

	for (int i=0; i<ThreadsNumber; i++)
		logfile << (LPCTSTR)span[i].Format(" %H:%M:%S");
	logfile << std::endl;
	logfile.flush();
}

int _tmain(int argc, _TCHAR* argv[])
{
	try {
		CreateArrays();

		logfile.open("DCOMvsNET.log",std::ios::out|std::ios::app);
		if (logfile.is_open())
		{
			logfile << "Start testing..." << std::endl;

			NETObjectMethod1Call(0);
			logfile << "NET call" << std::endl;
			logfile << "Method1"; RunMethod(NETObjectMethod1Call);
			logfile << "Method2"; RunMethod(NETObjectMethod2Call);
			logfile << "Method3"; RunMethod(NETObjectMethod3Call);
			logfile << "Method4"; RunMethod(NETObjectMethod4Call);
			logfile << "Method5"; RunMethod(NETObjectMethod5Call);

			logfile << "NET connect" << std::endl;
			logfile << "Method1"; RunMethod(NETObjectMethod1Connect);
			logfile << "Method2"; RunMethod(NETObjectMethod2Connect);
			logfile << "Method3"; RunMethod(NETObjectMethod3Connect);
			logfile << "Method4"; RunMethod(NETObjectMethod4Connect);
			logfile << "Method5"; RunMethod(NETObjectMethod5Connect);

			DCOMObjectMethod1Call(0);
			logfile << "DCOM call" << std::endl;
			logfile << "Method1"; RunMethod(DCOMObjectMethod1Call);
			logfile << "Method2"; RunMethod(DCOMObjectMethod2Call);
			logfile << "Method3"; RunMethod(DCOMObjectMethod3Call);
			logfile << "Method4"; RunMethod(DCOMObjectMethod4Call);
			logfile << "Method5"; RunMethod(DCOMObjectMethod5Call);

			logfile << "DCOM connect" << std::endl;
			logfile << "Method1"; RunMethod(DCOMObjectMethod1Connect);
			logfile << "Method2"; RunMethod(DCOMObjectMethod2Connect);
			logfile << "Method3"; RunMethod(DCOMObjectMethod3Connect);
			logfile << "Method4"; RunMethod(DCOMObjectMethod4Connect);
			logfile << "Method5"; RunMethod(DCOMObjectMethod5Connect);

			COMplusObjectMethod1Call(0);
			logfile << "COM+ call" << std::endl;
			logfile << "Method1"; RunMethod(COMplusObjectMethod1Call);
			logfile << "Method2"; RunMethod(COMplusObjectMethod2Call);
			logfile << "Method3"; RunMethod(COMplusObjectMethod3Call);
			logfile << "Method4"; RunMethod(COMplusObjectMethod4Call);
			logfile << "Method5"; RunMethod(COMplusObjectMethod5Call);

			logfile << "COM+ connect" << std::endl;
			logfile << "Method1"; RunMethod(COMplusObjectMethod1Connect);
			logfile << "Method2"; RunMethod(COMplusObjectMethod2Connect);
			logfile << "Method3"; RunMethod(COMplusObjectMethod3Connect);
			logfile << "Method4"; RunMethod(COMplusObjectMethod4Connect);
			logfile << "Method5"; RunMethod(COMplusObjectMethod5Connect);

			COMplusPooledObjectMethod1Call(0);
			logfile << "COM+ pooled call" << std::endl;
			logfile << "Method1"; RunMethod(COMplusPooledObjectMethod1Call);
			logfile << "Method2"; RunMethod(COMplusPooledObjectMethod2Call);
			logfile << "Method3"; RunMethod(COMplusPooledObjectMethod3Call);
			logfile << "Method4"; RunMethod(COMplusPooledObjectMethod4Call);
			logfile << "Method5"; RunMethod(COMplusPooledObjectMethod5Call);

			logfile << "COM+ pooled connect" << std::endl;
			logfile << "Method1"; RunMethod(COMplusPooledObjectMethod1Connect);
			logfile << "Method2"; RunMethod(COMplusPooledObjectMethod2Connect);
			logfile << "Method3"; RunMethod(COMplusPooledObjectMethod3Connect);
			logfile << "Method4"; RunMethod(COMplusPooledObjectMethod4Connect);
			logfile << "Method5"; RunMethod(COMplusPooledObjectMethod5Connect);

			logfile.close();
		}
	} catch (int hr) {
		if (logfile.is_open())
			logfile << "Error: HRESULT = 0x" << std::hex << hr << std::endl;
	} catch (...) {
	}

	return 0;
}
