using System;
using System.IO;
using System.Runtime.Remoting;
using System.Runtime.Remoting.Channels;
using System.Runtime.Remoting.Channels.Tcp;
using System.Runtime.Remoting.Channels.Http;
using System.Threading;
using System.Web.Mail;

namespace NETClient
{
	class Test
	{
		const int
			IterationsNumber = 1000,
			WarmingNumber = 5,
			ThreadsNumber = 10,
			ArraySize = 100;
		const string 
			TestString = "0123456789",
			ServerName = "localhost";

		static int[]    intArr = CreateIntArray();
		static string[] strArr = CreateStringArray();

		internal static int[] CreateIntArray()
		{
			int[] ar = new int[ArraySize];
			for (int i=0; i<ArraySize; i++)
				ar[i] = i;
			return ar;
		}

		internal static string[] CreateStringArray()
		{
			string [] ar = new string[ArraySize];
			for (int i=0; i<ArraySize; i++)
				ar[i] = TestString;
			return ar;
		}

		internal static TimeSpan WSObjectMethod1Stateful()
		{
			DateTime begin = DateTime.Now;
			localhost.WSObject ws = new localhost.WSObject();
			for (int i=0; i<IterationsNumber; i++)
			{
				ws.Method1();
			}
			return DateTime.Now-begin;
		}

		internal static TimeSpan WSObjectMethod1Connect()
		{
			DateTime begin = DateTime.Now;
			for (int i=0; i<IterationsNumber; i++)
			{
				localhost.WSObject ws = new localhost.WSObject();
				ws.Method1();
			}
			return DateTime.Now-begin;
		}

		internal static TimeSpan WSObjectMethod2Stateful()
		{
			DateTime begin = DateTime.Now;
			localhost.WSObject ws = new localhost.WSObject();
			int par = 0;
			for (int i=0; i<IterationsNumber; i++)
			{
				ws.Method2(par++);
			}
			return DateTime.Now-begin;
		}

		internal static TimeSpan WSObjectMethod2Connect()
		{
			DateTime begin = DateTime.Now;
			int par = 0;
			for (int i=0; i<IterationsNumber; i++)
			{
				localhost.WSObject ws = new localhost.WSObject();
				ws.Method2(par);
			}
			return DateTime.Now-begin;
		}

		internal static TimeSpan WSObjectMethod3Stateful()
		{
			DateTime begin = DateTime.Now;
			localhost.WSObject ws = new localhost.WSObject();
			string par = TestString;
			for (int i=0; i<IterationsNumber; i++)
			{
				ws.Method3(par);
			}
			return DateTime.Now-begin;
		}

		internal static TimeSpan WSObjectMethod3Connect()
		{
			DateTime begin = DateTime.Now;
			string par = TestString;
			for (int i=0; i<IterationsNumber; i++)
			{
				localhost.WSObject ws = new localhost.WSObject();
				ws.Method3(par);
			}
			return DateTime.Now-begin;
		}

		internal static TimeSpan WSObjectMethod4Stateful()
		{
			DateTime begin = DateTime.Now;
			localhost.WSObject ws = new localhost.WSObject();
			int[] par = intArr;
			for (int i=0; i<IterationsNumber; i++)
			{
				ws.Method4(par);
			}
			return DateTime.Now-begin;
		}

		internal static TimeSpan WSObjectMethod4Connect()
		{
			DateTime begin = DateTime.Now;
			int[] par = intArr;
			for (int i=0; i<IterationsNumber; i++)
			{
				localhost.WSObject ws = new localhost.WSObject();
				ws.Method4(par);
			}
			return DateTime.Now-begin;
		}

		internal static TimeSpan WSObjectMethod5Stateful()
		{
			DateTime begin = DateTime.Now;
			localhost.WSObject ws = new localhost.WSObject();
			string [] par = strArr;
			for (int i=0; i<IterationsNumber; i++)
			{
				ws.Method5(par);
			}
			return DateTime.Now-begin;
		}

		internal static TimeSpan WSObjectMethod5Connect()
		{
			DateTime begin = DateTime.Now;
			string [] par = strArr;
			for (int i=0; i<IterationsNumber; i++)
			{
				localhost.WSObject ws = new localhost.WSObject();
				ws.Method5(par);
			}
			return DateTime.Now-begin;
		}

		internal static TimeSpan RemMethod1Stateful(TestCreator creator)
		{
			DateTime begin = DateTime.Now;
			RemoteObject.Test o = creator();
			for (int i=0; i<IterationsNumber; i++)
			{
				o.Method1();
			}
			return DateTime.Now-begin;
		}

		internal static TimeSpan RemMethod1Connect(TestCreator creator)
		{
			DateTime begin = DateTime.Now;
			for (int i=0; i<IterationsNumber; i++)
			{
				RemoteObject.Test o = creator();
				o.Method1();
			}
			return DateTime.Now-begin;
		}

		internal static TimeSpan RemMethod2Stateful(TestCreator creator)
		{
			DateTime begin = DateTime.Now;
			RemoteObject.Test o = creator();
			int par = 0;
			for (int i=0; i<IterationsNumber; i++)
			{
				o.Method2(par++);
			}
			return DateTime.Now-begin;
		}

		internal static TimeSpan RemMethod2Connect(TestCreator creator)
		{
			DateTime begin = DateTime.Now;
			int par = 0;
			for (int i=0; i<IterationsNumber; i++)
			{
				RemoteObject.Test o = creator();
				o.Method2(par);
			}
			return DateTime.Now-begin;
		}

		internal static TimeSpan RemMethod3Stateful(TestCreator creator)
		{
			DateTime begin = DateTime.Now;
			RemoteObject.Test o = creator();
			string par = TestString;
			for (int i=0; i<IterationsNumber; i++)
			{
				o.Method3(par);
			}
			return DateTime.Now-begin;
		}

		internal static TimeSpan RemMethod3Connect(TestCreator creator)
		{
			DateTime begin = DateTime.Now;
			string par = TestString;
			for (int i=0; i<IterationsNumber; i++)
			{
				RemoteObject.Test o = creator();
				o.Method3(par);
			}
			return DateTime.Now-begin;
		}

		internal static TimeSpan RemMethod4Stateful(TestCreator creator)
		{
			DateTime begin = DateTime.Now;
			RemoteObject.Test o = creator();
			int[] par = intArr;
			for (int i=0; i<IterationsNumber; i++)
			{
				o.Method4(par);
			}
			return DateTime.Now-begin;
		}

		internal static TimeSpan RemMethod4Connect(TestCreator creator)
		{
			DateTime begin = DateTime.Now;
			int[] par = intArr;
			for (int i=0; i<IterationsNumber; i++)
			{
				RemoteObject.Test o = creator();
				o.Method4(par);
			}
			return DateTime.Now-begin;
		}

		internal static TimeSpan RemMethod5Stateful(TestCreator creator)
		{
			DateTime begin = DateTime.Now;
			RemoteObject.Test o = creator();
			string [] par = strArr;
			for (int i=0; i<IterationsNumber; i++)
			{
				o.Method5(par);
			}
			return DateTime.Now-begin;
		}

		internal static TimeSpan RemMethod5Connect(TestCreator creator)
		{
			DateTime begin = DateTime.Now;
			string [] par = strArr;
			for (int i=0; i<IterationsNumber; i++)
			{
				RemoteObject.Test o = creator();
				o.Method5(par);
			}
			return DateTime.Now - begin;
		}


		public delegate RemoteObject.Test TestCreator();
		public delegate TimeSpan RunMethod1();
		public delegate TimeSpan RunMethod2(TestCreator creator);

		static RemoteObject.Test TcpBinSingletonCreator()
		{ return new RemoteObject.TcpBinSingletonTest(); }
		static RemoteObject.Test TcpBinSingleCallCreator()
		{ return new RemoteObject.TcpBinSingleCallTest(); }
		static RemoteObject.Test TcpBinClientCreator()
		{ return new RemoteObject.TcpBinClientTest(); }

		static RemoteObject.Test TcpSoapSingletonCreator()
		{ return new RemoteObject.TcpSoapSingletonTest(); }
		static RemoteObject.Test TcpSoapSingleCallCreator()
		{ return new RemoteObject.TcpSoapSingleCallTest(); }
		static RemoteObject.Test TcpSoapClientCreator()
		{ return new RemoteObject.TcpSoapClientTest(); }

		static RemoteObject.Test HttpBinSingletonCreator()
		{ return new RemoteObject.HttpBinSingletonTest(); }
		static RemoteObject.Test HttpBinSingleCallCreator()
		{ return new RemoteObject.HttpBinSingleCallTest(); }
		static RemoteObject.Test HttpBinClientCreator()
		{ return new RemoteObject.HttpBinClientTest(); }

		static RemoteObject.Test HttpSoapSingletonCreator()
		{ return new RemoteObject.HttpSoapSingletonTest(); }
		static RemoteObject.Test HttpSoapSingleCallCreator()
		{ return new RemoteObject.HttpSoapSingleCallTest(); }
		static RemoteObject.Test HttpSoapClientCreator()
		{ return new RemoteObject.HttpSoapClientTest(); }

		static RemoteObject.Test IISHttpBinSingletonCreator()
		{ return new RemoteObject.IISHttpBinSingletonTest(); }
		static RemoteObject.Test IISHttpBinSingleCallCreator()
		{ return new RemoteObject.IISHttpBinSingleCallTest(); }
		static RemoteObject.Test IISHttpBinClientCreator()
		{ return new RemoteObject.IISHttpBinClientTest(); }

		static RemoteObject.Test IISHttpSoapSingletonCreator()
		{ return new RemoteObject.IISHttpSoapSingletonTest(); }
		static RemoteObject.Test IISHttpSoapSingleCallCreator()
		{ return new RemoteObject.IISHttpSoapSingleCallTest(); }
		static RemoteObject.Test IISHttpSoapClientCreator()
		{ return new RemoteObject.IISHttpSoapClientTest(); }

		struct RegType
		{
			internal RegType(Type tp,TestCreator c,string u)
			{
				url = u;
				type = tp;
				creator = c;
			}

			internal Type type;
			internal TestCreator creator;
			internal string url;
		}

		struct RegChannel
		{
			internal RegChannel(IChannel ch,RegType[] t)
			{
				channel = ch;
				types = t;
			}

			internal IChannel channel;
			internal RegType[] types;
		}

		static RegChannel[] reg = new RegChannel[]
		{
			new RegChannel(
				new TcpChannel(null,new BinaryClientFormatterSinkProvider(),null),
				new RegType[] 
				{
					new RegType(
						typeof(RemoteObject.TcpBinSingletonTest),
						new TestCreator(TcpBinSingletonCreator),
						"tcp://"+ServerName+":8008/RemotingTest/TcpBinSingletonTest.rem"),
					new RegType(
						typeof(RemoteObject.TcpBinSingleCallTest),
						new TestCreator(TcpBinSingleCallCreator),
						"tcp://"+ServerName+":8008/RemotingTest/TcpBinSingleCallTest.rem"),
					new RegType(
						typeof(RemoteObject.TcpBinClientTest),
						new TestCreator(TcpBinClientCreator),
						"tcp://"+ServerName+":8008")
				}),
			/*new RegChannel(
				new TcpChannel(null,new SoapClientFormatterSinkProvider(),null),
				new RegType[]
				{
					new RegType(
						typeof(RemoteObject.TcpSoapSingletonTest),
						new TestCreator(TcpSoapSingletonCreator),
						"tcp://"+ServerName+":8008/RemotingTest/TcpSoapSingletonTest.soap"),
					new RegType(
						typeof(RemoteObject.TcpSoapSingleCallTest),
						new TestCreator(TcpSoapSingleCallCreator),
						"tcp://"+ServerName+":8008/RemotingTest/TcpSoapSingleCallTest.soap"),
					new RegType(
						typeof(RemoteObject.TcpSoapClientTest),
						new TestCreator(TcpSoapClientCreator),
						"tcp://"+ServerName+":8008")
				}),
			new RegChannel(
				new HttpChannel(null,new BinaryClientFormatterSinkProvider(),null),
				new RegType[]
				{
					new RegType(
						typeof(RemoteObject.HttpBinSingletonTest),
						new TestCreator(HttpBinSingletonCreator),
						"http://"+ServerName+":8009/RemotingTest/HttpBinSingletonTest.rem"),
					new RegType(
						typeof(RemoteObject.HttpBinSingleCallTest),
						new TestCreator(HttpBinSingleCallCreator),
						"http://"+ServerName+":8009/RemotingTest/HttpBinSingleCallTest.rem"),
					new RegType(
						typeof(RemoteObject.HttpBinClientTest),
						new TestCreator(HttpBinClientCreator),
						"http://"+ServerName+":8009")
				}),
			*/new RegChannel(
				new HttpChannel(null,new SoapClientFormatterSinkProvider(),null),
				new RegType[]
				{
					new RegType(
						typeof(RemoteObject.HttpSoapSingletonTest),
						new TestCreator(HttpSoapSingletonCreator),
						"http://"+ServerName+":8009/RemotingTest/HttpSoapSingletonTest.soap"),
					new RegType(
						typeof(RemoteObject.HttpSoapSingleCallTest),
						new TestCreator(HttpSoapSingleCallCreator),
						"http://"+ServerName+":8009/RemotingTest/HttpSoapSingleCallTest.soap"),
					new RegType(
						typeof(RemoteObject.HttpSoapClientTest),
						new TestCreator(HttpSoapClientCreator),
						"http://"+ServerName+":8009")
				}),
			new RegChannel(
				new HttpChannel(null,new BinaryClientFormatterSinkProvider(),null),
				new RegType[]
				{
					new RegType(
						typeof(RemoteObject.IISHttpBinSingletonTest),
						new TestCreator(IISHttpBinSingletonCreator),
						"http://"+ServerName+"/NETTest/IISHttpBinSingletonTest.rem"),
					new RegType(
						typeof(RemoteObject.IISHttpBinSingleCallTest),
						new TestCreator(IISHttpBinSingleCallCreator),
						"http://"+ServerName+"/NETTest/IISHttpBinSingleCallTest.rem"),
					new RegType(
						typeof(RemoteObject.IISHttpBinClientTest),
						new TestCreator(IISHttpBinClientCreator),
						"http://"+ServerName+"/NETTest/")
				}),
			new RegChannel(
				new HttpChannel(null,new SoapClientFormatterSinkProvider(),null),
				new RegType[]
				{
					new RegType(
						typeof(RemoteObject.IISHttpSoapSingletonTest),
						new TestCreator(IISHttpSoapSingletonCreator),
						"http://"+ServerName+"/NETTest/IISHttpSoapSingletonTest.soap"),
					new RegType(
						typeof(RemoteObject.IISHttpSoapSingleCallTest),
						new TestCreator(IISHttpSoapSingleCallCreator),
						"http://"+ServerName+"/NETTest/IISHttpSoapSingleCallTest.soap"),
					new RegType(
						typeof(RemoteObject.IISHttpSoapClientTest),
						new TestCreator(IISHttpSoapClientCreator),
						"http://"+ServerName+"/NETTest/")
				})
		};
		
		static void Register(RegChannel rc)
		{
			while (ChannelServices.RegisteredChannels.Length > 0)
				ChannelServices.UnregisterChannel(ChannelServices.RegisteredChannels[0]);

            ChannelServices.RegisterChannel(rc.channel);
			
			RemotingConfiguration.RegisterWellKnownClientType(rc.types[0].type,rc.types[0].url);
			RemotingConfiguration.RegisterWellKnownClientType(rc.types[1].type,rc.types[1].url);
			RemotingConfiguration.RegisterActivatedClientType(rc.types[2].type,rc.types[2].url);
		}

		public class ThreadRunner
		{
			RunMethod1 _method1;
			RunMethod2 _method2;
			TestCreator _creator;
			public Thread thread;
			public TimeSpan span;

			public ThreadRunner(RunMethod1 m1)
			{
				_method1 = m1;
				thread = new Thread(new ThreadStart(Thread1));
			}

			public ThreadRunner(RunMethod2 m2,TestCreator cr)
			{
				_method2 = m2;
				_creator = cr;
				thread = new Thread(new ThreadStart(Thread2));
			}

			void Thread1()
			{
				span = _method1();
			}

			void Thread2()
			{
				span = _method2(_creator);
			}
		}

		void RunRemMethod(TextWriter tw,RunMethod1 method)
		{
			// Warming up
			for (int i=0; i<WarmingNumber; i++)
			{
				localhost.WSObject ws = new localhost.WSObject();
				ws.Method1();
			}

            ThreadRunner[] th = new ThreadRunner[ThreadsNumber];
			for (int i=0; i<th.Length; i++)
				th[i] = new ThreadRunner(method);

			DateTime begin = DateTime.Now;

			for (int i=0; i<th.Length; i++)
				th[i].thread.Start();
			for (int i=0; i<th.Length; i++)
				th[i].thread.Join();

			TimeSpan span = DateTime.Now - begin;
			tw.Write(" {0}:",span);

			for (int i=0; i<th.Length; i++)
                tw.Write(" {0}",th[i].span);
			tw.WriteLine();
		}

		void RunRemMethod(TextWriter tw,TestCreator creator,RunMethod2 method)
		{
			// Warming up
			for (int i=0; i<WarmingNumber; i++)
			{
				RemoteObject.Test o = creator();
				o.Method1();
			}

			ThreadRunner[] th = new ThreadRunner[ThreadsNumber];
			for (int i=0; i<th.Length; i++)
				th[i] = new ThreadRunner(method,creator);

			DateTime begin = DateTime.Now;

			for (int i=0; i<th.Length; i++)
				th[i].thread.Start();
			for (int i=0; i<th.Length; i++)
				th[i].thread.Join();

			TimeSpan span = DateTime.Now - begin;
			tw.Write(" {0}:",span);

			for (int i=0; i<th.Length; i++)
				tw.Write(" {0}",th[i].span);
			tw.WriteLine();
		}

		void TestWS(TextWriter tw)
		{
			tw.WriteLine("Web-serveice stateful"); tw.Flush();
			tw.Write("Method1"); RunRemMethod(tw,new RunMethod1(WSObjectMethod1Stateful)); tw.Flush();
			tw.Write("Method2"); RunRemMethod(tw,new RunMethod1(WSObjectMethod2Stateful)); tw.Flush();
			tw.Write("Method3"); RunRemMethod(tw,new RunMethod1(WSObjectMethod3Stateful)); tw.Flush();
			tw.Write("Method4"); RunRemMethod(tw,new RunMethod1(WSObjectMethod4Stateful)); tw.Flush();
			tw.Write("Method5"); RunRemMethod(tw,new RunMethod1(WSObjectMethod5Stateful)); tw.Flush();

			tw.WriteLine("Web-serveice connect"); tw.Flush();
			tw.Write("Method1"); RunRemMethod(tw,new RunMethod1(WSObjectMethod1Connect)); tw.Flush();
			tw.Write("Method2"); RunRemMethod(tw,new RunMethod1(WSObjectMethod2Connect)); tw.Flush();
			tw.Write("Method3"); RunRemMethod(tw,new RunMethod1(WSObjectMethod3Connect)); tw.Flush();
			tw.Write("Method4"); RunRemMethod(tw,new RunMethod1(WSObjectMethod4Connect)); tw.Flush();
			tw.Write("Method5"); RunRemMethod(tw,new RunMethod1(WSObjectMethod5Connect)); tw.Flush();
		}

		void TestRem(TextWriter tw,TestCreator creator)
		{
			string str = creator().GetType().ToString();

			tw.WriteLine(str+" stateful"); tw.Flush();
			tw.Write("Method1"); RunRemMethod(tw,creator,new RunMethod2(RemMethod1Stateful)); tw.Flush();
			tw.Write("Method2"); RunRemMethod(tw,creator,new RunMethod2(RemMethod2Stateful)); tw.Flush();
			tw.Write("Method3"); RunRemMethod(tw,creator,new RunMethod2(RemMethod3Stateful)); tw.Flush();
			tw.Write("Method4"); RunRemMethod(tw,creator,new RunMethod2(RemMethod4Stateful)); tw.Flush();
			tw.Write("Method5"); RunRemMethod(tw,creator,new RunMethod2(RemMethod5Stateful)); tw.Flush();

			tw.WriteLine(str+" connect"); tw.Flush();
			tw.Write("Method1"); RunRemMethod(tw,creator,new RunMethod2(RemMethod1Connect)); tw.Flush();
			tw.Write("Method2"); RunRemMethod(tw,creator,new RunMethod2(RemMethod2Connect)); tw.Flush();
			tw.Write("Method3"); RunRemMethod(tw,creator,new RunMethod2(RemMethod3Connect)); tw.Flush();
			tw.Write("Method4"); RunRemMethod(tw,creator,new RunMethod2(RemMethod4Connect)); tw.Flush();
			tw.Write("Method5"); RunRemMethod(tw,creator,new RunMethod2(RemMethod5Connect)); tw.Flush();
		}

		[STAThread]
		static void Main(string[] args)
		{
			Test test = new Test();

			using (TextWriter tw = new StreamWriter("DCOMvsNET.log",true))
			{
				try 
				{
					tw.WriteLine("Start testing...");

					foreach (RegChannel rc in reg)
					{
						Register(rc);
						test.TestRem(tw,rc.types[0].creator);
						test.TestRem(tw,rc.types[1].creator);
						test.TestRem(tw,rc.types[2].creator);
					}
					test.TestWS(tw);

					tw.WriteLine("");
				} 
				catch (Exception ex) 
				{
					tw.WriteLine(ex.Message);
				}
			}

/*
			CDO.MessageClass msg = new CDO.MessageClass();
			msg.From = "igor.tkachev@stis.com";
			msg.To = "igor.tkachev@stis.com";
			msg.Subject = "DCOMvsNET";
			msg.Body = DateTime.Now.ToString();
			//msg.AddAttachment((new MailAttachment("DCOMvsNET.log"));
			CDO.SSmtpMail.SmtpServer = "fl-mia01-m";
			System.Web.Mail.SmtpMail.Send(msg);
*/			
		}
	}
}
